from flask import current_app
from werkzeug.exceptions import NotFound, BadRequest

from flaschengeist.models.user import User, Role
from flaschengeist.database import db
from flaschengeist import logger


def login_user(username, password):
    logger.info("login user {{ {} }}".format(username))
    user = User.query.filter(User.userid == username).one_or_none()
    if user is None:
        user = User(userid=username)
        db.session.add(user)
    if current_app.config["FG_AUTH_BACKEND"].login(user, password):
        update_user(user)
        return user
    return None


def update_user(user):
    current_app.config["FG_AUTH_BACKEND"].update_user(user)
    if not user.display_name:
        user.display_name = "{} {}".format(user.firstname, user.lastname)
    db.session.commit()


def set_roles(user: User, roles: [str]):
    user.roles.clear()
    for role_name in roles:
        role = Role.query.filter(Role.name == role_name).one_or_none()
        if not role:
            raise BadRequest("Role not found >{}<".format(role_name))
        user.roles.append(role)


def modify_user(user, password, new_password=None):
    """Modify given user on the backend

    Args:
        user: User object to sync with backend
        password: Current password (most backends are needing this)
        new_password (optional): New password, if password should be changed

    Raises:
        NotImplemented: If backend is not capable of this operation
        BadRequest: Password is wrong or other logic issues
    """
    current_app.config["FG_AUTH_BACKEND"].modify_user(user, password, new_password)


def get_users():
    return User.query.all()


def get_user_by_role(role: Role):
    return User.query.join(User.roles).filter_by(role_id=role.id).all()


def get_user(uid):
    user = User.query.filter(User.userid == uid).one_or_none()
    if not user:
        raise NotFound
    return user


def delete(user):
    current_app.config["FG_AUTH_BACKEND"].delete_user(user)
    db.session.delete(user)
    db.session.commit()
